% Script file: ErroreCTMN.m
%
% Analisi dell'errore per l'equazione del calore 
%            u_t = u_xx,  x in [0,L] 
% risolta con il theta-metodo.
% Lo script utilizza le funzioni 
% un1=CEPasso(un,dt,dx,val) e un1=CIPasso(un,dt,dx,val)
% Il dato iniziale e' 
%               u(x,0)=cos(pi*x/L)
% con condizioni al bordo di Neumann u_x(0,t)=0, u_x(L,t)=0.
% Il problema ammette la soluzione esatta
%               u(x,t)=exp(-t*pi^2/L^2)*cos(pi*x/L)
%

close all;
clear; clc;
L = 1;
Ne = 200 ;
xe=linspace(0,L,Ne);
n0=5;
N0 = 20 ;
dx0 = L/(N0-1);
theta=input('Inserisci il valore di theta in [0,1] :');
a = 2*(1-2*theta) ;
if theta >= 1/2
   a = dx0 ;
end
dt0 = dx0^2/a ;
t = n0*dt0 ;
disp('[1] Condizioni al bordo di ordine uno in dx');
disp('[2] Condizioni al bordo di ordine due in dx');
bc=input('Scegli lo schema al contorno (1,2) :');
disp(sprintf('Tempo finale di calcolo = %f', t));
kmax = 4 ;
passo = 8 ;
n = n0 ;
N = N0 ;
E=zeros(kmax,8*(N0-1)+1);

% Ripete kmax volte i calcoli con numero di punti crescente 

for k=1:kmax
   dx = L/(N-1);
   if theta >= 1/2
      a = dx ;
   end
   dt = dx^2 / a ;
   x=linspace(0,L,N);
   u=cos(pi*x/L);
   ue=cos(pi*xe/L);
   u=u';
   plot(x,u,'o',xe,ue,'r')
   xlabel('x')
   ylabel('u(x,t)')
   s=sprintf('Distribuzione iniziale per N=%1.0f:',N); 
   title(s);
   pause(1)
   tc = 0 ;
   lambda = 1/a;
   
   % condizioni sulla matrice dello schema implicito 
   
   tl = theta*lambda;
   if bc == 1
      bl = [1,0,0,0, 1+tl,-tl];
      br = [-tl, 1+tl,0,0,0,1];
   elseif bc == 2
      bl = [1+2*tl,-2*tl,0,-tl,1+2*tl,-tl];
      br = [-tl,1+2*tl,-tl,0,-2*tl,1+2*tl];
   end      

   % Calcola l'evoluzione al tempo t
      
   for h=0:n
      tc = tc+dt;
      if bc == 2
         u1=u(1)+2*(1-theta)*lambda*(u(2)-u(1));
         uN=u(N)+2*(1-theta)*lambda*(u(N-1)-u(N));
      end      
      u=CEPasso(u,(1-theta)*dt,dx);
      if bc == 2
         u(1)=u1;
         u(N)=uN;
      end
      u=CIPasso(u,theta*dt,dx,bl,br);
      if bc == 1
         u(1)=u(2);
         u(N)=u(N-1);
      end      
      ue=exp(-tc*pi^2/L^2)*cos(pi*xe/L);
      plot(x,u,'o',xe,ue,'r')
      s=sprintf('Evoluzione al tempo t=%f per N=%1.0f',tc,N);
      title(s);
      xlabel('x');
      ylabel('u(x,t)');
      axis([0 1 -1 1]);
      pause(0.1)
   end
   ue=exp(-tc*pi^2/L^2)*cos(pi*x/L);
   E(k,1:passo:8*(N0-1)+1)=abs(ue-u)/abs(ue);
   n=n*2;
   
   % Se theta < 1/2 i punti crescono in modo quadratico 
   
   if theta < 1/2
      n = n*2;
   end
   N=2*N-1;
   passo = passo/2;
end

% Realizza il grafico dell'errore

figure
passo=8;
N=N0;
hold on
for k=1:kmax
   x=linspace(0,L,N);
   Errore(k)=max(E(k,1:passo:8*(N0-1)+1));
   plot(x,E(k,1:passo:8*(N0-1)+1),'-o');
   passo = passo /2;
   N=2*N-1;
   s=sprintf('Errore al tempo finale t=%f',tc);
   title(s);
   xlabel('x');
   ylabel('E(x,t)');
end
hold off

% Tabella con errore massimo e ordine di convergenza

disp(sprintf('EQUAZIONE DEL CALORE: theta = %f',theta));
disp(' --------------------------------'); 
disp(sprintf('|N \t| Errore\t| Ordine\t|')); 
disp(' --------------------------------'); 
N=N0;
for k=1:kmax-1
   s=sprintf('|%1.0f\t| %3.2e\t|\t\t\t|',N,Errore(k));    
   disp(s);
   ordine(k)= log2(Errore(k)/Errore(k+1));
   s=sprintf('|\t|\t\t\t|\t%3.2f\t|',ordine(k));
   disp(s);
   N=2*N-1;
end
s=sprintf('|%1.0f\t| %3.2e\t|\t\t\t|',N,Errore(kmax));
disp(s);
