% Script file: Poisson2D.m
%
% Risoluzione dell'equazione di Poisson in 2D 
%      u_xx + u_yy = f(x,y),  (x,y) in [0,L]x[0,L]
% tramite le differenze centrali.
% Lo script utilizza le funzioni un1=JCPasso2D(un,f,dx) e
% un1=GSPasso2D(un,f,dx)
% Inizialmente si assume u(x,y)=0 ed il problema e dato da 
%               f(x,y)=1   
% con condizioni al bordo u(x,y)=0.

close all;
clear; clc;
L = 1;
N = 40;
dx = L/(N-1);
mt= input('Jacobi o Gauss-Seidel  (1 o 2) :');
n = input('Inserisci numero di iterazioni : ');
disp('[1] Griglia');
disp('[2] Superficie');
disp('[3] Contorni');
disp('[4] Rendering');
scelta = input('Scegli il tipo di grafico : ');
u=zeros(N,N);
f=ones(N,N);
x=linspace(0,L,N);
y=linspace(0,L,N);
for nc=1:n
   un=u;
   if mt==1
      u=JCPasso(u,f,dx);
   elseif mt==2
      u=GSPasso(u,f,dx);
   end
   if max(max(abs(u-un))) < 0.000001
      break;
   end
   disp(sprintf('Iterazione n. %d Min=%f',nc,min(min(u))));
end

switch scelta
case 1
   mesh(x,y,u);
case 2
   surf(x,y,u);
case 3
   contourf(x,y,u);
case 4
   surf(x,y,u);
   shading interp;
   colormap(jet);
   camlight right;
   lighting phong;
end
s=sprintf('Soluzione dopo n=%d iterazioni',n);
title(s);
xlabel('x')
ylabel('y')
if scelta ~= 3 
   zlabel('u(x,y)');
end
